// Filename: commandline.c
// Name of author: Petr Kovar
// Work begun:    10/22/2002
// Last modified: 04/01/2007
//
// Description:
// parser for simple one character commanline options

/*
Copyright (C) 2006 Petr Kov
You can reach me at Petr.Kovar@vsb.cz

-------------------------------------

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

// ------------------------------------------------------------------------------------------------------------------------

// Library
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

// Hlavikov soubory
#include "resitele_types.h"
#include "commandline.h"
#include "messages.h"

// constructor for parameter structure
PARAMETERS * new_parameters(void) {
  PARAMETERS * p_parameters;         // global parameters
  time_t now;                        // raw time variable
  struct tm tm_var;                  // structuralized time variable

  // allocate memory
  p_parameters = (PARAMETERS *) malloc(sizeof(PARAMETERS));

  p_parameters->verbosity_level = 3;                 // level of printed comments
  p_parameters->info_archiv = 2;                     // jak podrobn pst informace o souboru loh
  p_parameters->info_resitele = 2;                   // jak podrobn pst informace o souboru eitel
  p_parameters->info_body = 3;                       // jak podrobn pst informace do souboru s archivem
  p_parameters->info_ulohy = 2;                      // jak podrobn zapisovat nzvy loh do tabulky

  p_parameters->in_file_resitele = "resitele.txt";   // name of the file for the input with resitele
  p_parameters->in_file_ulohy = "ulohy.txt";         // name of the file for the input with ulohy
  p_parameters->out_file_resitele = "resitele.php";  // name of the file for the output with resitele
  p_parameters->out_file_archiv = "archiv.php";      // name of the file for the output with archive
  p_parameters->out_file_statistika = "statistika.php";  // name of the file for the output with statistika

  p_parameters->archiv_head = "archiv_head.php";     // name of the file with 'archiv' headers
  p_parameters->archiv_tail = "archiv_tail.php";     // name of the file with 'archiv' tails

  p_parameters->resitele_head = "resitele_head.php"; // name of the file with 'resitele' headers
  p_parameters->resitele_mids = "resitele_mid.php";  // name of the file with 'resitele' headers
  p_parameters->resitele_tail = "resitele_tail.php"; // name of the file with 'resitele' tails

  p_parameters->statistika_head = "statistika_head.php";  // name of the file with 'statistika' headers
  p_parameters->statistika_tail = "statistika_tail.php";  // name of the file with 'statistika' tails
  // set date to now
  time(&now);                                        // snapshot of system clock
  tm_var = *localtime(&now);                         // structuralize
  p_parameters->datum.den = tm_var.tm_mday;          // day is not important    (not +1 !)
  p_parameters->datum.mesic = tm_var.tm_mon+1;       // for table of this month (+1 !)
  p_parameters->datum.rok = tm_var.tm_year+1900;     // for table of this year (+1900 !)

  return p_parameters;
}

// free the PARAMETERS structure
int free_parameters(PARAMETERS * p_parameters) {
  if (p_parameters != NULL) {
    // free the structure
    free(p_parameters);
  }
  return OK;
}

// print how to use this program
void print_usage(void) {
  // Tisk npovdy
  printf("Pouit:\n");
  printf("resitele [-m <number>] [-y <number>] [-h] [-v <number>]\n");
  printf("         [-U <number>] [-R <number>] [U <number>]\n");
  printf("         -r <text file resitelu> -u <text file uloh>\n");
  printf("         -t <php file resitelu> -a <php file archivu>\n");
  printf(" -h  ... tisk tto npovdy\n");
  printf(" -m  ... msc (tento msc)\n");
  printf(" -y  ... rok (tento rok)\n");
  printf(" -v  ... podrobnost iformac (3) [1-3]\n");
  //D  printf(" -z  ... pouh zmna tabulek v existujcch *.html souborech za aktualizovan tabulky\n");
  printf(" -U ... informace o dekodovn souboru loh (2) [1-3]\n");
  printf(" -R ... informace o dekodovn souboru eitel (2) [1-3]\n");
  printf(" -B ... informace o bodech za lohy (2) [1-4]\n");
  printf(" -T ... informace zapisovan do tabulky eitel (2) [1-4]\n");
  printf(" -u ... jmno soubou loh\n");
  printf(" -r ... jmno soubou eitel\n");
  printf(" -a ... jmno vstupnho souboru s tabulkou archivu\n");
  printf(" -t ... jmno vstupnho souboru s tabulkou eitel\n");
  //D  printf(" -iz ... informace o zmn archivnho souboru a souboru eitel\n");
  printf(" Sytaxe <text file resitelu>:\n");
  printf("   {den.mesic.rok}{jmeno}{uloha}{body}\n");
  printf(" Sytaxe <text file uloh>:\n");
  printf("   {archiv.cislo.}{jmeno ulohy}{body}\n");
}

// go through all parameters, check them and store to * p_parameters
int scan_parameters(int argc, char ** argv, PARAMETERS * p_parameters) {
  int i = 1;                        // go through the parameter list
  unsigned int j;                   // go through the parameter shortcut list
  int k;                            // the parameters to convert
  char c;                           // parameter 
  char jmeno_souboru[MAX_RADEK];    // maximln dlka stringu

  // go through the parameter list, we may skip something
  while (i < argc) {
    // is this a parameter?
    if (*argv[i] != '-') {
      // no, a missmatch!
      print_usage();
    }
    else {
      // next parameter to convert
      k = 1;
      // yes go through them
      for (j = 1; j < strlen(argv[i]); j++) {
	c = argv[i][j];
	switch(c) {
	case 'h':
	  print_usage();
	  exit(OK);
	  break;
	case 'm':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%u", &(p_parameters->datum.mesic)) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'y':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%u", &(p_parameters->datum.rok)) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'U':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%u", &(p_parameters->info_archiv)) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'R':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%u", &(p_parameters->info_resitele)) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'B':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%u", &(p_parameters->info_body)) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'T':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%u", &(p_parameters->info_ulohy)) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'r':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%s", jmeno_souboru) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  p_parameters->in_file_resitele = (char *) malloc(sizeof(char)*strlen(jmeno_souboru)+1);
	  strcpy(p_parameters->in_file_resitele, jmeno_souboru);
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'u':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%s", jmeno_souboru) != 1)) { // static !!!
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  p_parameters->in_file_ulohy = (char *) malloc(sizeof(char)*strlen(jmeno_souboru)+1);
	  strcpy(p_parameters->in_file_ulohy, jmeno_souboru);
	  // move to the next parameter to scan
	  k++;
	  break;
	case 'a':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%s", jmeno_souboru) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  p_parameters->out_file_archiv = (char *) malloc(sizeof(char)*strlen(jmeno_souboru)+1);
	  strcpy(p_parameters->out_file_archiv, jmeno_souboru);
	  // move to the next parameter to scan
	  k++;
	  break;
	case 't':
	  if ((i+k >= argc) || (sscanf(argv[i+k], "%s", jmeno_souboru) != 1)) {
	    print_usage();
	    return BAD_PARAMETERS;
	  }
	  p_parameters->out_file_resitele = (char *) malloc(sizeof(char)*strlen(jmeno_souboru)+1);
	  strcpy(p_parameters->out_file_resitele, jmeno_souboru);
	  // move to the next parameter to scan
	  k++;
	  break;
	default:
	  print_usage();
	  return BAD_PARAMETERS;
	}
      }
    }
    // k arguments for one parameter
    i += k;
  }
  // done
  return OK;
}

// ------------------------------------------------------------------------------------------------------------------------

// end of file
