// Filename: resitele.c
// Name of author: Petr Kovar
// Work begun:    28/12/2004
// Last modified: 08/12/2007
//
// Description:
// file with program for generating tables for 'ulohy'

/*
Copyright (C) 2007 Petr Kov
You can reach me at Petr.Kovar@vsb.cz

-------------------------------------

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

// ------------------------------------------------------------------------------------------------------------------------

// Verze
#define VERSION    "3.8.1"

// Standardn knihovny
#include <stdio.h>
#include <stdlib.h>

// Definice typ
#include "resitele_types.h"
#include "commandline.h"
#include "resitele.h"
#include "statistika.h"
#include "messages.h"
#include "main.h"

// this is the main function
int main(int argc, char * argv[]) {
  PARAMETERS * parameters;                   // globln parametery
  SEZNAM_ULOH * seznam_uloh;                 // seznam vech loh ze souboru
  SEZNAM_RESENI * seznam_reseni;             // seznam vech een ze souboru
  SEZNAM_RESITELU * seznam_resitelu;         // seznam vech eitel
  SEZNAM_RESITELU * seznam_resitelu_mesice;  // seznam eitel aktulnho msce

  // Parametery pkazov dky
  parameters = new_parameters();
  // jsou njak parametry pkazov dky?
  if (argc > 1) {
    if (scan_parameters(argc, argv, parameters) == BAD_PARAMETERS) {
      exit(BAD_PARAMETERS);
    }
  }
  // Hallo
  if (parameters->verbosity_level > 1) {
    fprintf(stdout, "Program 'resitele' (verze %s)\n", VERSION);
  }

  // nati strukturu loh ze souboru
  seznam_uloh = read_ulohy(parameters);
  // vypi seznam loh
  //T  if (parameters->verbosity_level > 1) {
  if (parameters->verbosity_level > 3) {
    print_seznam_uloh(seznam_uloh);
  }

  // Nov struktura eitel
  seznam_resitelu = new_seznam_resitelu();
  // nati strukturu odpovd ze souboru a vytvo seznam een i databzi eitel
  seznam_reseni = read_odpovedi(parameters, seznam_uloh, seznam_resitelu);

  // vypi seznam odpovd
  //T  if (parameters->verbosity_level > 1) {
  if (parameters->verbosity_level > 3) {
    print_seznam_reseni(seznam_reseni);
  }
  // vypi seznam eitel
  //T  if (parameters->verbosity_level > 1) {
  if (parameters->verbosity_level > 3) {
    print_seznam_resitelu(seznam_resitelu);
  }

  // Archiv
  create_archive_file(parameters, seznam_uloh);
  // upodej eitele podle potu bod
  counting_sort_resitele_by_points(seznam_resitelu);

  // eitel msce
  seznam_resitelu_mesice = resitele_mesice(seznam_resitelu, parameters->datum.mesic, parameters->datum.rok);
  //D  print_seznam_resitelu(seznam_resitelu_mesice);
  // upodej eitele msce podle potu bod
  counting_sort_resitele_by_points(seznam_resitelu_mesice);

  // tabulka eitel
  //T  create_resitele_file(parameters, seznam_resitelu, seznam_resitelu_mesice);
  create_resitele_file(parameters, seznam_resitelu, seznam_resitelu_mesice);

  // resitele mezi daty, ulohy mezi daty -> mesic i rok a pod....
  // ??

  // statistika
  create_statistika_file(parameters, seznam_reseni, seznam_uloh, seznam_resitelu);
  // !!

  // info o aktulnch lohch
  aktualni_ulohy(parameters, seznam_uloh);

  // free memory
  free_seznam_uloh(seznam_uloh);
  free_seznam_resitelu(seznam_resitelu);
  free_seznam_resitelu(seznam_resitelu_mesice);
  free_seznam_reseni(seznam_reseni);

  // Hotovo se vm
  if (parameters->verbosity_level > 2) {
    fprintf(stdout, "Konec.\n");
  }

  // free also parameters ;o)
  free_parameters(parameters);

  // done
  return(OK);
}

// ------------------------------------------------------------------------------------------------------------------------

// end of file
